﻿using Buddy.Coroutines;
using ff14bot;
using ff14bot.AClasses;
using ff14bot.Enums;
using ff14bot.Managers;
using ff14bot.Objects;
using System;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using TreeSharp;

namespace stealthy
{
    class Stealthy : BotPlugin
    {
        private TreeSharp.Composite Root;
        private bool useMount = true;
        public override string Author
        {
            get
            {
                return "Eniac";
            }
        }

        public override string Name
        {
            get
            {
                return "Stealthy";
            }
        }

        public static readonly Version version = new Version(1, 0, 8);
        public override Version Version
        {
            get
            {
                return version;
            }
        }

        override
        public void OnInitialize()
        {
            useMount = ff14bot.Settings.CharacterSettings.Instance.UseMount;
        }

        override
        public void OnEnabled()
        {
            Root = new ActionRunCoroutine(r => MainStealthLogic());
            Root.Start(null);
        }

        override
        public void OnDisabled()
        {
            Root.Stop(null);
            Root = null;
            ff14bot.Settings.CharacterSettings.Instance.UseMount = useMount;

        }

        override
        public void OnShutdown()
        {
            ff14bot.Settings.CharacterSettings.Instance.UseMount = useMount;
        }

        override
        public void OnPulse()
        {
            try
            {
                Root.Tick(null);
                if (Root.LastStatus != TreeSharp.RunStatus.Running)
                {
                    Root.Stop(null);
                    Root.Start(null);
                }
            }
            catch (Exception e)
            {
                ff14bot.Helpers.Logging.WriteException(e);
                Root.Stop(null);
                Root.Start(null);
                throw;
            }
        }

        private async Task MainStealthLogic()
        {
            if (Core.Player.CurrentJob == ff14bot.Enums.ClassJobType.Botanist || Core.Player.CurrentJob == ff14bot.Enums.ClassJobType.Miner)
            {
                if (GameObjectManager.GetObjectsOfType<BattleCharacter>().Where((u, i) => u.StatusFlags.HasFlag(StatusFlags.Hostile) &&
                                                                                          u.ClassLevel >= Core.Me.ClassLevel - 10 &&
                                                                                          u.Location.Distance(Core.Player.Location) < 30).FirstOrDefault() != null)
                {
                    ff14bot.Settings.CharacterSettings.Instance.UseMount = false;
                    if (Core.Player.IsMounted)
                    {
                        ff14bot.Managers.Actionmanager.Dismount();
                    }
                    await Coroutine.Wait(Timeout.Infinite, () => !Core.Player.IsMounted);
                    SpellData data;
                    if (!Core.Me.HasAura("Stealth") && Actionmanager.CurrentActions.TryGetValue("Stealth", out data) && Actionmanager.CanCast(data, Core.Me))
                    {
                        ff14bot.Helpers.Logging.Write("Enabling stealth");
                        Actionmanager.DoAction("Stealth", Core.Me);
                    }
                    await Coroutine.Wait(Timeout.Infinite, () => Core.Me.HasAura("Stealth"));
                }
                else
                {
                    ff14bot.Settings.CharacterSettings.Instance.UseMount = true;
                    SpellData data;
                    if (Core.Me.HasAura("Stealth") && Actionmanager.CurrentActions.TryGetValue("Stealth", out data) && Actionmanager.CanCast(data, Core.Me))
                    {
                        ff14bot.Helpers.Logging.Write("Disabling stealth");
                        Actionmanager.DoAction("Stealth", Core.Me);
                    }
                    await Coroutine.Wait(Timeout.Infinite, () => !Core.Me.HasAura("Stealth"));
                }
            }
        }
    }
}
